package org.netbeans.lib.cvsclient.file;

import org.netbeans.lib.cvsclient.util.*;

/**
 * @author Thomas Singer
 */
public final class RepositoryFileNameParser {

	// Constants ==============================================================

	private static final String REPOSITORY_FILE_SUFFIX = ",v";

	// Fields =================================================================

	private final ICvsFileSystem cvsFileSystem;
	private final String relativeRepositoryRootPath;

	// Setup ==================================================================

	public RepositoryFileNameParser(String relativeRepositoryRootPath, ICvsFileSystem cvsFileSystem) {
		BugLog.assertNotNull(relativeRepositoryRootPath);
		BugLog.assertNotNull(cvsFileSystem);
		this.relativeRepositoryRootPath = relativeRepositoryRootPath;
		this.cvsFileSystem = cvsFileSystem;
	}

	// Accessing ==============================================================

	public FsFilePath parse(String directory, String absoluteRcsFileName) throws ParseException {
		if (!absoluteRcsFileName.endsWith(REPOSITORY_FILE_SUFFIX)) {
			throw new ParseException("'" + absoluteRcsFileName + "' does not end with '" + REPOSITORY_FILE_SUFFIX + "'");
		}

		if (!directory.startsWith(relativeRepositoryRootPath)) {
			throw new ParseException("directory '" + directory + "' does not start with '" + relativeRepositoryRootPath + "'");
		}

		final String relativeDirectoryPath = directory.substring(relativeRepositoryRootPath.length());
		final String fileName = getFileName(absoluteRcsFileName);
		return FsFilePath.createInstance(relativeDirectoryPath + fileName);
	}

	/**
	 * @deprecated does not work correctly when the files are stored differently on the server than they are expected by the client
	 *             Example: '/cvsroot/g/gr/gruntspud/gruntspud/.cvsignore,v' does not start with '/cvsroot/gruntspud/'
	 */
	public FsFilePath parse(String absoluteRcsFileName) throws ParseException {
		if (!absoluteRcsFileName.endsWith(REPOSITORY_FILE_SUFFIX)) {
			throw new ParseException("'" + absoluteRcsFileName + "' does not end with '" + REPOSITORY_FILE_SUFFIX + "'");
		}

		final String relativeRepositoryPath = commaVRepositoryFilePathToRelativeFile(cvsFileSystem.getRelativeRepositoryPath(absoluteRcsFileName));
		if (!relativeRepositoryPath.startsWith(relativeRepositoryRootPath)) {
			throw new ParseException("'" + relativeRepositoryPath + "' does not start with '" + relativeRepositoryRootPath + "'");
		}

		return FsFilePath.createInstance(relativeRepositoryPath.substring(relativeRepositoryRootPath.length()));
	}

	// Utils ==================================================================

	private static String commaVRepositoryFilePathToRelativeFile(String relativeRepositoryPath) {
		relativeRepositoryPath = relativeRepositoryPath.substring(0, relativeRepositoryPath.length() - REPOSITORY_FILE_SUFFIX.length());
		final int lastSlashIndex = relativeRepositoryPath.lastIndexOf('/');
		if (lastSlashIndex >= 0) {
			final int secondLastSlashIndex = relativeRepositoryPath.lastIndexOf('/', lastSlashIndex - 1);
			if (secondLastSlashIndex >= 0) {
				final String lastDirectory = relativeRepositoryPath.substring(secondLastSlashIndex, lastSlashIndex);
				if (lastDirectory.equals("/Attic")) {
					relativeRepositoryPath = relativeRepositoryPath.substring(0, secondLastSlashIndex) + relativeRepositoryPath.substring(lastSlashIndex);
				}
			}
		}
		return relativeRepositoryPath;
	}

	private String getFileName(String absoluteRcsFileName) {
		final int lastSlashIndex = absoluteRcsFileName.lastIndexOf('/');
		if (lastSlashIndex < 0) {
			return null;
		}

		return absoluteRcsFileName.substring(lastSlashIndex + 1, absoluteRcsFileName.length() - REPOSITORY_FILE_SUFFIX.length());
	}

	// Inner classes ==========================================================

	public static final class ParseException extends Exception {
		private ParseException(String message) {
			super(message);
		}
	}
}
